"use strict";

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllVersionsInfo = getAllVersionsInfo;
exports.getAllVersionsObjects = getAllVersionsObjects;
exports.getAllVersionHashesByVersionsObjects = getAllVersionHashesByVersionsObjects;
exports.getAllVersionHashes = getAllVersionHashes;
exports.hasVersionByRef = hasVersionByRef;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

/**
 * the snaps are saved as DAG (Direct Acyclic Graph).
 * each snap has `parents` prop.
 * when this is the first snap, the `parents` is empty.
 * the followed snap has the first snap as a parent.
 * in case of a merge between lanes, the `parents` have two snaps from the two lanes.
 *
 * traverse all versions is not cheap. it must load the Version object to extract the `parents`
 * data. so, we plan to cache it. once this is cached, we'll change the implementation of a few
 * methods here.
 */

/**
 * by default it starts the traverse from the head or lane-head, unless "startFrom" is passed.
 * if versionObjects passed, use it instead of loading from the repo.
 */
function getAllVersionsInfo(_x) {
  return _getAllVersionsInfo.apply(this, arguments);
}

function _getAllVersionsInfo() {
  _getAllVersionsInfo = (0, _bluebird().coroutine)(function* ({
    modelComponent,
    repo,
    throws = true,
    versionObjects,
    startFrom
  }) {
    const results = [];

    const getVersionObj = /*#__PURE__*/function () {
      var _ref = (0, _bluebird().coroutine)(function* (ref) {
        if (versionObjects) return versionObjects.find(v => v.hash().isEqual(ref));
        if (repo) return yield ref.load(repo);
        throw new TypeError('getAllVersionsInfo expect to get either repo or versionObjects');
      });

      return function getVersionObj(_x12) {
        return _ref.apply(this, arguments);
      };
    }();

    const getRefToStartFrom = () => {
      if (typeof startFrom !== 'undefined') return startFrom;
      return modelComponent.laneHeadLocal || modelComponent.getHead();
    };

    const laneHead = getRefToStartFrom();

    if (laneHead) {
      const headInfo = {
        ref: laneHead,
        tag: modelComponent.getTagOfRefIfExists(laneHead)
      };
      const head = yield getVersionObj(laneHead);

      if (head) {
        headInfo.version = head;
      } else {
        headInfo.error = new (_exceptions().HeadNotFound)(modelComponent.id(), laneHead.toString());
        if (throws) throw headInfo.error;
      }

      results.push(headInfo);

      const addParentsRecursively = /*#__PURE__*/function () {
        var _ref2 = (0, _bluebird().coroutine)(function* (version) {
          yield Promise.all(version.parents.map( /*#__PURE__*/function () {
            var _ref3 = (0, _bluebird().coroutine)(function* (parent) {
              const parentVersion = yield getVersionObj(parent);
              const versionInfo = {
                ref: parent,
                tag: modelComponent.getTagOfRefIfExists(parent)
              };

              if (parentVersion) {
                versionInfo.version = parentVersion;
                yield addParentsRecursively(parentVersion);
              } else {
                versionInfo.error = versionInfo.tag ? new (_exceptions().VersionNotFound)(versionInfo.tag) : new (_exceptions().ParentNotFound)(modelComponent.id(), version.hash().toString(), parent.toString());
                if (throws) throw versionInfo.error;
              }

              results.push(versionInfo);
            });

            return function (_x14) {
              return _ref3.apply(this, arguments);
            };
          }()));
        });

        return function addParentsRecursively(_x13) {
          return _ref2.apply(this, arguments);
        };
      }();

      if (head) yield addParentsRecursively(head);
    } // backward compatibility.
    // components created before v15, might not have head.
    // even if they do have head (as a result of tag/snap after v15), they
    // have old versions without parents and new versions with parents


    yield Promise.all(Object.keys(modelComponent.versions).map( /*#__PURE__*/function () {
      var _ref4 = (0, _bluebird().coroutine)(function* (version) {
        if (!results.find(r => r.tag === version)) {
          const ref = modelComponent.versions[version];
          const versionObj = yield getVersionObj(ref);
          const versionInfo = {
            ref,
            tag: version
          };
          if (versionObj) versionInfo.version = versionObj;else {
            versionInfo.error = new (_exceptions().VersionNotFound)(version);
            if (throws) throw versionInfo.error;
          }
          results.push(versionInfo);
        }
      });

      return function (_x15) {
        return _ref4.apply(this, arguments);
      };
    }()));
    return results;
  });
  return _getAllVersionsInfo.apply(this, arguments);
}

function getAllVersionsObjects(_x2, _x3) {
  return _getAllVersionsObjects.apply(this, arguments);
}

function _getAllVersionsObjects() {
  _getAllVersionsObjects = (0, _bluebird().coroutine)(function* (modelComponent, repo, throws = true) {
    const allVersionsInfo = yield getAllVersionsInfo({
      modelComponent,
      repo,
      throws
    });
    return allVersionsInfo.map(a => a.version).filter(a => a);
  });
  return _getAllVersionsObjects.apply(this, arguments);
}

function getAllVersionHashesByVersionsObjects(_x4, _x5) {
  return _getAllVersionHashesByVersionsObjects.apply(this, arguments);
}

function _getAllVersionHashesByVersionsObjects() {
  _getAllVersionHashesByVersionsObjects = (0, _bluebird().coroutine)(function* (modelComponent, versionObjects, throws = true) {
    const allVersionsInfo = yield getAllVersionsInfo({
      modelComponent,
      throws,
      versionObjects
    });
    return allVersionsInfo.map(v => v.ref).filter(ref => ref);
  });
  return _getAllVersionHashesByVersionsObjects.apply(this, arguments);
}

function getAllVersionHashes(_x6, _x7) {
  return _getAllVersionHashes.apply(this, arguments);
}

function _getAllVersionHashes() {
  _getAllVersionHashes = (0, _bluebird().coroutine)(function* (modelComponent, repo, throws = true, startFrom) {
    const allVersionsInfo = yield getAllVersionsInfo({
      modelComponent,
      repo,
      throws,
      startFrom
    });
    return allVersionsInfo.map(v => v.ref).filter(ref => ref);
  });
  return _getAllVersionHashes.apply(this, arguments);
}

function hasVersionByRef(_x8, _x9, _x10, _x11) {
  return _hasVersionByRef.apply(this, arguments);
}

function _hasVersionByRef() {
  _hasVersionByRef = (0, _bluebird().coroutine)(function* (modelComponent, ref, repo, startFrom) {
    const allVersionHashes = yield getAllVersionHashes(modelComponent, repo, true, startFrom);
    return allVersionHashes.some(hash => hash.isEqual(ref));
  });
  return _hasVersionByRef.apply(this, arguments);
}